
# q-expansion of newform 1155.2.c.c, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1155
weight = 2
poly_data = [2, -4, 4, 2, 2, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[38, 2, -1, -4, 8, -1], 23], [[29, 10, -5, -20, 17, -5], 23], [[-13, 32, 30, 5, -10, 7], 23], [[27, -70, -11, -21, 19, -11], 23], [[26, -64, -37, -10, 20, -14], 23]]

hecke_ring_character_values = [[386, [1, 0, 0, 0, 0, 0]], [232, [-1, 0, 0, 0, 0, 0]], [661, [1, 0, 0, 0, 0, 0]], [211, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, -1], [0, 0, 0, -1, 0, 0], [0, 0, -1, 0, 0, -1], [0, 0, 0, -1, 0, 0], [-1, 0, 0, 0, 0, 0], [0, 0, 0, -1, -1, 0], [0, 0, 0, 0, -1, 1], [-1, 0, -2, 0, 0, 0], [0, 0, 0, 0, -1, -1], [2, -2, 1, 0, 0, 0], [-4, 1, -4, 0, 0, 0], [0, 0, 0, -1, -1, 0], [-3, 0, -1, 0, 0, 0], [0, 0, 0, -2, 1, -2], [0, 0, 0, 0, 2, -1], [0, 0, 0, 7, 0, 2], [5, -1, 0, 0, 0, 0], [-8, -3, -1, 0, 0, 0], [0, 0, 0, -2, 0, 0], [-3, -4, 3, 0, 0, 0], [0, 0, 0, -8, 0, -2], [0, -1, 4, 0, 0, 0], [0, 0, 0, 8, 3, -1], [6, -2, -3, 0, 0, 0], [0, 0, 0, 0, -1, 4], [-6, 0, -4, 0, 0, 0], [0, 0, 0, -4, 5, -4], [0, 0, 0, -6, -2, -1], [2, 3, -4, 0, 0, 0], [0, 0, 0, 8, 1, -5], [0, 0, 0, 1, -4, -1], [-1, 2, 1, 0, 0, 0], [0, 0, 0, 1, 3, -7], [-1, 1, 9, 0, 0, 0], [-10, 0, 0, 0, 0, 0], [0, 2, 2, 0, 0, 0], [0, 0, 0, -1, -4, -3], [0, 0, 0, -1, -1, 0], [0, 0, 0, -5, 3, -5], [0, 0, 0, -1, -3, -9], [11, -4, 13, 0, 0, 0], [-2, 8, 6, 0, 0, 0], [2, 2, 10, 0, 0, 0], [0, 0, 0, 8, -10, 2], [0, 0, 0, 8, -10, 10], [6, 2, -8, 0, 0, 0], [-6, -6, 6, 0, 0, 0], [0, 0, 0, -4, 11, -8], [0, 0, 0, 1, 10, -6], [-8, -3, -4, 0, 0, 0], [0, 0, 0, 8, -10, 7], [-19, 3, -2, 0, 0, 0], [-17, -7, -1, 0, 0, 0], [0, 12, 4, 0, 0, 0], [0, 0, 0, -18, 6, -7], [0, 0, 0, 14, 0, 4], [-3, 1, -12, 0, 0, 0], [-4, 9, 5, 0, 0, 0], [0, 0, 0, 25, 1, -5], [-13, -5, 1, 0, 0, 0], [0, 0, 0, -3, 1, -10], [0, 0, 0, -11, 10, -8], [0, 0, 0, 12, 2, -8], [-8, -12, 4, 0, 0, 0], [0, 0, 0, -7, -6, 11], [0, 0, 0, 13, -9, 1], [11, -10, 2, 0, 0, 0], [0, 0, 0, 4, -3, 0], [0, 0, 0, -6, 0, -8], [1, 12, -12, 0, 0, 0], [0, 0, 0, 4, 2, 12], [-2, -4, -1, 0, 0, 0], [0, 0, 0, 9, 6, -13], [0, 0, 0, -7, -6, 3], [0, 7, 7, 0, 0, 0], [0, 0, 0, -14, -2, -1], [5, -6, 9, 0, 0, 0], [0, 0, 0, -15, 1, -1], [-1, -10, -7, 0, 0, 0], [7, 1, 7, 0, 0, 0], [12, 0, -11, 0, 0, 0], [-7, -6, 0, 0, 0, 0], [8, -6, 16, 0, 0, 0], [0, 0, 0, -4, 18, 6], [-9, 2, -10, 0, 0, 0], [0, 0, 0, 13, -7, -3], [11, -6, -3, 0, 0, 0], [0, 0, 0, -14, -1, -6], [14, -8, -6, 0, 0, 0], [0, 0, 0, -26, 10, 0], [0, 0, 0, 14, 4, -4], [21, -6, -1, 0, 0, 0], [0, 0, 0, -2, 18, 4], [1, -5, -14, 0, 0, 0], [22, -9, 15, 0, 0, 0], [0, 0, 0, 1, -2, 4], [12, -16, 13, 0, 0, 0], [7, -13, 6, 0, 0, 0], [0, 0, 0, -14, -4, -2], [-2, 9, 14, 0, 0, 0], [0, 0, 0, -5, 20, 1], [0, 0, 0, -8, -2, -12], [0, 0, 0, -1, 7, 7], [5, -15, -12, 0, 0, 0], [8, 7, -12, 0, 0, 0], [0, 0, 0, 24, 6, 8], [0, 0, 0, -26, -4, 3], [0, 0, 0, 1, -1, 3], [-6, -2, -12, 0, 0, 0], [6, 5, -5, 0, 0, 0], [0, 0, 0, -6, -6, -16], [0, 0, 0, -6, 8, 18], [0, 0, 0, -15, -1, 21], [-2, 4, 12, 0, 0, 0], [16, -1, -11, 0, 0, 0], [4, 2, 14, 0, 0, 0], [0, 0, 0, -15, -8, 19], [0, 0, 0, 12, -20, 6], [0, 0, 0, 3, 2, 10], [-13, 17, -2, 0, 0, 0], [4, -13, -12, 0, 0, 0], [0, 0, 0, -5, -2, 15], [0, 0, 0, 6, -11, -9], [0, 0, 0, 32, -4, -4], [-12, -2, -22, 0, 0, 0], [-2, -12, -11, 0, 0, 0], [34, -3, 1, 0, 0, 0], [19, -13, 22, 0, 0, 0], [0, 0, 0, -5, 10, -11], [0, 0, 0, 22, -22, 4], [-8, -16, -6, 0, 0, 0], [0, 0, 0, -16, 8, -12], [-11, 0, 4, 0, 0, 0], [0, 0, 0, -32, 16, -10], [-30, 6, -18, 0, 0, 0], [-1, -2, -14, 0, 0, 0], [0, 0, 0, 22, -32, 14], [0, 0, 0, 18, -14, -2], [0, 0, 0, -20, -14, 4], [12, -6, 14, 0, 0, 0], [2, 24, -6, 0, 0, 0], [-21, 13, -18, 0, 0, 0], [0, 0, 0, -5, -21, 2], [0, 0, 0, -30, -8, -4], [16, 4, 12, 0, 0, 0], [4, 2, 15, 0, 0, 0], [0, 0, 0, -5, -1, -8], [0, 0, 0, 10, -16, -8], [-20, 13, -22, 0, 0, 0], [0, 0, 0, -3, -16, 24], [0, 0, 0, 12, 15, 14], [-24, -6, -7, 0, 0, 0], [0, 0, 0, -8, 26, -8], [0, 0, 0, 25, -6, -4], [0, 0, 0, -13, 5, -16], [-10, 4, -14, 0, 0, 0], [6, 4, -6, 0, 0, 0], [1, -19, 7, 0, 0, 0], [0, 0, 0, 27, -5, -4], [9, -14, 13, 0, 0, 0], [0, 0, 0, 2, -21, 21], [0, 0, 0, 6, -16, -4], [0, 0, 0, -2, 11, -14], [6, -6, 19, 0, 0, 0], [0, 0, 0, 21, -12, 18], [0, 0, 0, 0, -20, 8], [-3, -10, -14, 0, 0, 0], [0, 0, 0, 8, 10, 6], [14, 7, -14, 0, 0, 0], [0, 0, 0, -8, 2, -5], [-34, 6, -9, 0, 0, 0], [-2, 21, 0, 0, 0, 0], [22, -2, -1, 0, 0, 0], [0, 0, 0, -15, 12, 13], [-16, -4, -6, 0, 0, 0], [16, -12, -18, 0, 0, 0], [-12, -13, 18, 0, 0, 0], [-14, -14, 6, 0, 0, 0], [0, 0, 0, -2, -10, -12], [-23, 5, -31, 0, 0, 0], [0, 0, 0, -10, -20, 8], [-2, -22, 4, 0, 0, 0], [0, 0, 0, 4, -4, 14], [0, 0, 0, -7, 14, 12], [0, 0, 0, 15, -6, -18], [15, 11, 14, 0, 0, 0], [0, 0, 0, 29, 6, 5], [0, 0, 0, 17, -26, 17], [-26, -19, 2, 0, 0, 0], [-9, 2, -11, 0, 0, 0], [0, 0, 0, -32, -2, -14], [0, 0, 0, 24, 2, -22], [1, 9, 3, 0, 0, 0], [15, -1, -6, 0, 0, 0], [0, 0, 0, -14, 26, -9], [0, 0, 0, -34, 20, -2], [12, 20, -14, 0, 0, 0], [0, 0, 0, -3, 11, -26], [0, 0, 0, 5, 4, 20], [0, 0, 0, -7, 13, 9], [-31, 15, -4, 0, 0, 0], [45, 6, 6, 0, 0, 0], [0, 0, 0, 32, 1, -16], [27, 5, 7, 0, 0, 0], [2, 14, -7, 0, 0, 0], [0, 0, 0, 32, -20, -4], [14, 1, -2, 0, 0, 0], [0, 0, 0, -20, 34, -26], [-2, 18, -23, 0, 0, 0], [34, -7, 22, 0, 0, 0], [0, 0, 0, -17, 6, 9], [-24, -4, -5, 0, 0, 0], [0, 0, 0, 18, 10, 8], [0, 0, 0, -12, 6, 1], [-16, 0, -4, 0, 0, 0], [-24, -4, -2, 0, 0, 0], [0, 0, 0, -8, -17, 12], [4, 4, -26, 0, 0, 0], [0, 0, 0, 5, -15, 7], [0, 0, 0, -8, 19, -25], [-10, -10, 16, 0, 0, 0], [-35, -7, 3, 0, 0, 0], [13, 8, 5, 0, 0, 0], [0, 0, 0, 21, -5, -18], [0, 0, 0, -39, 4, 14], [26, -7, 34, 0, 0, 0], [0, 0, 0, 4, -28, -7], [23, 8, 7, 0, 0, 0], [0, 0, 0, 12, -4, 26], [29, 3, -12, 0, 0, 0], [0, 0, 0, 15, -11, 4], [-7, -17, -1, 0, 0, 0], [18, -19, 6, 0, 0, 0], [-29, -15, 0, 0, 0, 0], [0, 0, 0, 1, -28, 15], [0, 0, 0, -13, 20, -20], [-16, 19, -26, 0, 0, 0], [0, 0, 0, 28, -4, 2], [0, 26, 4, 0, 0, 0], [6, 16, -22, 0, 0, 0], [0, 0, 0, -33, 7, -1], [20, 29, -1, 0, 0, 0], [0, 0, 0, -9, -31, -19], [-2, -23, 15, 0, 0, 0], [0, 0, 0, -13, -4, 18], [25, -2, -11, 0, 0, 0], [0, 0, 0, 8, 8, 10], [-15, -13, 0, 0, 0, 0], [18, -25, 20, 0, 0, 0], [0, 0, 0, 42, -10, 0], [0, 0, 0, -22, -12, 16], [-3, -4, 1, 0, 0, 0], [0, 0, 0, 22, -30, 8], [0, 23, -14, 0, 0, 0], [0, 0, 0, 3, -6, 12], [4, 16, -23, 0, 0, 0], [24, 31, -3, 0, 0, 0], [0, 0, 0, 27, -35, 6], [0, 0, 0, 6, -4, 15], [0, 0, 0, 52, 8, -14], [0, 0, 0, -9, -4, 19], [0, 0, 0, -14, 36, 2], [11, -8, 12, 0, 0, 0], [0, 0, 0, 27, -17, 2], [0, 0, 0, -26, -18, -11], [14, 1, 6, 0, 0, 0], [-32, 10, 6, 0, 0, 0], [-30, 0, -23, 0, 0, 0], [0, 0, 0, 43, -4, -9], [0, 0, 0, -2, 12, -33], [0, 6, 6, 0, 0, 0], [0, 0, 0, 38, -4, 12], [0, 0, 0, 15, 22, 7], [-24, -19, 10, 0, 0, 0], [0, 0, 0, 15, -13, 12], [0, 0, 0, -45, -3, 5], [0, 0, 0, -21, 8, 4], [-12, -24, 6, 0, 0, 0], [-8, 23, -34, 0, 0, 0], [2, 28, 20, 0, 0, 0], [0, 0, 0, -9, 18, -10], [-1, 1, -15, 0, 0, 0], [0, 0, 0, 2, 14, -16], [-34, 7, -23, 0, 0, 0], [0, 0, 0, 29, -13, 18], [19, -30, 21, 0, 0, 0], [0, 0, 0, 25, 19, -8], [0, 0, 0, 16, -12, 12], [0, -18, 4, 0, 0, 0], [21, 6, -17, 0, 0, 0], [10, -20, 32, 0, 0, 0], [0, 0, 0, 22, 12, -7], [0, 0, 0, -22, 10, 23], [8, 0, -8, 0, 0, 0], [0, 0, 0, 10, -4, -26], [34, 8, 12, 0, 0, 0], [-18, 4, 0, 0, 0, 0], [0, 0, 0, -28, -18, -15], [14, 24, 12, 0, 0, 0], [0, 0, 0, 22, -40, 0], [0, 0, 0, -50, 16, 2], [0, 0, 0, -34, 32, -5], [-8, -35, 8, 0, 0, 0]]
